using System;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Windows.Forms;
 
namespace FolderContextMenu
{
    /// <summary>
    ///   Add-in Express Add-in Module
    /// </summary>
    [GuidAttribute("BB118371-246F-4865-BDD8-4DF14509CC39"), ProgId("FolderContextMenu.AddinModule"), RunInstaller(true)]
    public class AddinModule : AddinExpress.MSO.ADXAddinModule
    {
        public AddinModule(System.ComponentModel.IContainer container)
        {
            // This call is required by the Component Designer
            container.Add(this);
            InitializeComponent();
 
            // TODO: Add any initialization after the InitializeComponent call
        }

		private AddinExpress.MSO.ADXOutlookAppEvents adxOutlookEvents;
		private AddinExpress.MSO.ADXCommandBar adxCommandBar = null;
 
        public AddinModule()
        {
            InitializeComponent();
        }
 
        // Add-in Module Implementation
 
        #region Component Designer generated code
        /// <summary>
        /// Required by designer
        /// </summary>
        private System.ComponentModel.IContainer components;
 
        /// <summary>
        /// Required by designer support - do not modify
        /// the following method
        /// </summary>
        private void InitializeComponent()
        {
			this.components = new System.ComponentModel.Container();
			this.adxOutlookEvents = new AddinExpress.MSO.ADXOutlookAppEvents(this.components);
			// 
			// adxOutlookEvents
			// 
			this.adxOutlookEvents.CommandBarsUpdate += new System.EventHandler(this.adxOutlookEvents_CommandBarsUpdate);
			// 
			// AddinModule
			// 
			this.AddinName = "Context Menu Example";
			this.SupportedApps = AddinExpress.MSO.ADXOfficeHostApp.ohaOutlook;

		}
        #endregion
 
        #region ADX automatic code
 
        // Required by Add-in Express - do not modify
        // the methods within this region
 
        public override System.ComponentModel.IContainer GetContainer()
        {
            return components;
        }
 
        [ComRegisterFunctionAttribute]
        public static void AddinRegister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
        }
 
        [ComUnregisterFunctionAttribute]
        public static void AddinUnregister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
        }
 
        public override void UninstallControls()
        {
            base.UninstallControls();
        }

        #endregion

		internal Outlook._Application OutlookApp
		{
			get
			{
				return (HostApplication as Outlook._Application);
			}
		}

		private void adxOutlookEvents_CommandBarsUpdate(object sender, System.EventArgs e)
		{
			DoCommandBarsUpdate(this);
		}
 
		private void DoClick(object sender)
		{
			MessageBox.Show((sender as AddinExpress.MSO.ADXCommandBarControl).Caption, AddinName);
		}

		private void DoOpenOptionPage(object sender)
		{
			MyPropertyPage properties = null;
			try
			{
				properties = new MyPropertyPage();
				if (properties.ShowDialog() == DialogResult.OK)
				{
				}
			}
			finally
			{
				if (properties != null)
					properties.Dispose();
			}
		}

		internal bool IsInboxContextMenu(Office.CommandBar contextBar)
		{
			for(int i=1; i<=contextBar.Controls.Count; i++)
			{
				if (contextBar.Controls[i].Caption == "&Copy \"Inbox\"...")
					return true;
			}
			return false;
		}

		internal AddinExpress.MSO.ADXCommandBarControl ControlByControlTag(string controlTag)
		{
			for(int i=0; i<adxCommandBar.Controls.Count; i++)
			{
				if (adxCommandBar.Controls[i].ControlTag == controlTag)
					return adxCommandBar.Controls[i];
			}
			return null;
		}

		internal void AddControl(Office.CommandBar contextBar, string caption, string controlTag, AddinExpress.MSO.ADXClick_EventHandler eventHandler)
		{
			AddinExpress.MSO.ADXCommandBarButton adxNewButton = null;
			Office.CommandBarControl newButton = 
				contextBar.FindControl(Office.MsoControlType.msoControlButton, Type.Missing, controlTag, Type.Missing, Type.Missing) as Office.CommandBarControl;
			if (newButton == null)
			{
				contextBar.Protection = Office.MsoBarProtection.msoBarNoProtection;
				if (adxCommandBar == null)
				{
					adxCommandBar = new AddinExpress.MSO.ADXCommandBar();
					adxCommandBar.CommandBarObj = contextBar;
				}
				else
				{
					adxNewButton = ControlByControlTag(controlTag) as AddinExpress.MSO.ADXCommandBarButton;
					if (adxNewButton != null)
					{
						adxNewButton.ControlObj = null;
						adxCommandBar.Controls.RemoveAt(adxCommandBar.Controls.IndexOf(adxNewButton));
					}
				}
				adxNewButton = new AddinExpress.MSO.ADXCommandBarButton();
				adxNewButton.ControlTag = controlTag;
				adxNewButton.Before = 0;
				adxNewButton.Temporary = true;
				adxNewButton.BeginGroup = true;
				adxNewButton.Style = AddinExpress.MSO.ADXMsoButtonStyle.adxMsoButtonCaption;
				adxNewButton.Caption = caption;
				adxNewButton.Click += eventHandler;
				if (adxCommandBar.Controls.Add(adxNewButton) != null)
				{
					adxNewButton.ControlObj = contextBar.Controls.Add(Office.MsoControlType.msoControlButton, Type.Missing, Type.Missing, Type.Missing, true);
					adxNewButton.UpdateControlObj(this);
				}
			}
			else
			{
				Marshal.ReleaseComObject(newButton);
			}
		}

		internal void DoCommandBarsUpdate(object sender)
		{
			Outlook.NameSpace nameSpace = OutlookApp.GetNamespace("MAPI");
			Outlook.Explorer activeExplorer = OutlookApp.ActiveExplorer();
			if (activeExplorer != null && nameSpace != null)
			{
				Outlook.MAPIFolder defaultFolder = nameSpace.GetDefaultFolder(Outlook.OlDefaultFolders.olFolderInbox);
				Outlook.MAPIFolder currentFolder = activeExplorer.CurrentFolder;
				if (currentFolder != null && defaultFolder != null)
				{
					if (currentFolder.EntryID == defaultFolder.EntryID)
					{
						this.CommandBarsObj = activeExplorer.CommandBars;	
						Office.CommandBar contextBar = this.FindCommandBarObj("Folder Context Menu") as Office.CommandBar;
						if (contextBar != null)
						{
							if (contextBar.Controls.Count > 0)
							{
								try
								{
									if (IsInboxContextMenu(contextBar))
									{
										AddControl(contextBar, "Test Button", "MY_UNIQUE_TAG 1", new AddinExpress.MSO.ADXClick_EventHandler(DoClick));
										AddControl(contextBar, "My Option Page", "MY_UNIQUE_TAG 2", new AddinExpress.MSO.ADXClick_EventHandler(DoOpenOptionPage));
									}
									else
										contextBar.Reset();
								}
								catch
								{
								}
							}
							Marshal.ReleaseComObject(contextBar);
						}
						this.CommandBarsObj = null;
					}
					Marshal.ReleaseComObject(defaultFolder);
					Marshal.ReleaseComObject(currentFolder);
				}
				Marshal.ReleaseComObject(nameSpace);
				Marshal.ReleaseComObject(activeExplorer);
			}
		}
	}
}

